#include <iostream>
#include <cstdio>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <pcap.h>

#include <siena/types.h>

#include <sffNetMessage/sffpcap.h>
#include <sffNetMessage/net_message.h>
#include <sffNetMessage/parser_inst.h>
#include <sffNetMessage/parser_types.h>

#include "tcpdump/setsignal.h"

using namespace std;

// --- forward declarations

void ether_if_print(u_char *user, const struct pcap_pkthdr *h, const u_char *p);
static RETSIGTYPE cleanup(int signo);


// --- siena::string_t cout operator

inline ostream & operator << (ostream & os, const string_t & x) {
       return os << std::string(x.begin, x.end - x.begin);
}


// ----- MAIN

int main(void) {

	printf("main: welcome to the test\n");

	ParserInst::staticInit();

// ---
	int	count = -1;
	u_char *pcap_userdata = 0;
	RETSIGTYPE (*oldhandler)(int);
	u_int32_t localnet, netmask;

// --- setup pcap

	pcap_t* pd = sffpcap_device_init(NULL, "", 1024, &localnet, &netmask);

// --- setup signal handlers if desired
	printf("main: setup signal handlers... ");
	(void)setsignal(SIGTERM, cleanup);
	(void)setsignal(SIGINT, cleanup);
	/* Cooperate with nohup(1) */
	if(SIG_DFL != (oldhandler = setsignal(SIGHUP, cleanup)) ) {
		(void)setsignal(SIGHUP, oldhandler);
	}
	printf("ok\n");

// --- start the callback loop

	sffpcap_run(pd, count, pcap_userdata, ether_if_print);

// ---

	printf("main: closed\n");

	return 0;
}

// --- SIGNAL STUFF

static RETSIGTYPE cleanup(int signo)
{
	printf("cleanup!\n");
	exit(0);
}

// --- pcap callback

void ether_if_print(u_char *user, const struct pcap_pkthdr *hdr, const u_char *data) {
	static	net_message	msg;

	if( ! msg.setup(eEther, user, hdr, data)) {
		return;
	}

cout << "New Message" <<endl;

	siena::message::iterator *iterator = msg.first();
	do {
		siena::string_t name = iterator->name();
		cout << " - " << name ;
		switch (iterator->type()) {
			case siena::bool_id: {
				bool bval = iterator->bool_value();
				cout << " - bool(" << bval << ")";
				break;
			}
			case siena::int_id: {
				int ival = iterator->int_value();
				cout << " - int(" << ival << ")";
				break;
			}
			case siena::double_id: {
				double dval = iterator->double_value();
				cout << " - double(" << dval << ")";
				break;
			}
			case siena::string_id: {
				siena::string_t sval = iterator->string_value();
				cout << " - string(" << sval << ")";
				break;
			}
		}
		cout << " - done\n";
	} while (iterator->next());
}
