#include <iostream>
#include <cstdio>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <pcap.h>

#include <sys/time.h>

#include <siena/types.h>
#include <siena/forwarding.h>

#include <sffNetMessage/sffpcap.h>
#include <sffNetMessage/net_message.h>
#include <sffNetMessage/parser_inst.h>
#include <sffNetMessage/parser_types.h>

#include "tcpdump/setsignal.h"

#include "callback.h"

using namespace std;

// --- globals *ick*

FwdTable *gFT = NULL;

struct timeval gTimeStart;
struct timeval gTimeStop;

// --- forward declarations

void ether_if_print(u_char *user, const struct pcap_pkthdr *h, const u_char *p);
static RETSIGTYPE cleanup(int signo);


// --- siena::string_t cout operator

inline ostream & operator << (ostream & os, const string_t & x) {
       return os << std::string(x.begin, x.end - x.begin);
}


// ----- MAIN

int sffnmSetupCallback(siena::FwdTable *FT, const char* file) {

//	printf("main: welcome to the test\n");

	ParserInst::staticInit();

// ---
	int	count = -1;
	u_char *pcap_userdata = 0;
	RETSIGTYPE (*oldhandler)(int);
	u_int32_t localnet, netmask;

// --- setup global

	gFT = FT;

// --- setup pcap

	pcap_t* pd = NULL;

	if(NULL == file) {
		pd = sffpcap_device_init(NULL, "", 0, &localnet, &netmask);
	} else {
		pd = sffpcap_file_init(file, "");
	}

// --- setup signal handlers if desired
//	printf("main: setup signal handlers... ");
	(void)setsignal(SIGTERM, cleanup);
	(void)setsignal(SIGINT, cleanup);
	/* Cooperate with nohup(1) */
	if(SIG_DFL != (oldhandler = setsignal(SIGHUP, cleanup)) ) {
		(void)setsignal(SIGHUP, oldhandler);
	}
//	printf("ok\n");

// --- start the timer

	gettimeofday(&gTimeStart, NULL);

// --- start the callback loop

	sffpcap_run(pd, count, pcap_userdata, ether_if_print);

// --- stop the timer

	gettimeofday(&gTimeStop, NULL);

// ---

//	printf("main: closed\n");

	return 0;
}

// --- SIGNAL STUFF

static RETSIGTYPE cleanup(int signo)
{
// --- stop the timer on interrupt

	gettimeofday(&gTimeStop, NULL);

	printf("cleanup!\n");

// --- print the stats.

	sffnmPrintStats();

	exit(0);
}

// ---

EtherMatchHandler	gMatchHandler;
int			gPackets = 0;

void sffnmPrintStats() {
	long long	start = gTimeStart.tv_usec + (gTimeStart.tv_sec * 1000000);
	long long	stop  = gTimeStop.tv_usec  + (gTimeStop.tv_sec  * 1000000);

	gMatchHandler.printStats();

	printf("Total Packets: %d\n", gPackets);
	printf("Time Elapsed (us): %lld\n", stop - start);
}

// --- pcap callback


void ether_if_print(u_char *user, const struct pcap_pkthdr *hdr, const u_char *data) {
	static	net_message	msg;
//	static	EtherMatchHandler	matchHandler;

	if( ! msg.setup(eEther, user, hdr, data)) {
		return;
	}

//cout << "New Message" <<endl;

	gPackets++;

	gMatchHandler.mMessage = &msg;

	gFT->match(msg, gMatchHandler);

/*
	siena::message::iterator *iterator = msg.first();
	do {
		siena::string_t name = iterator->name();
		cout << " - " << name ;
		switch (iterator->type()) {
			case siena::bool_id: {
				bool bval = iterator->bool_value();
				cout << " - bool(" << bval << ")";
				break;
			}
			case siena::int_id: {
				int ival = iterator->int_value();
				cout << " - int(" << ival << ")";
				break;
			}
			case siena::double_id: {
				double dval = iterator->double_value();
				cout << " - double(" << dval << ")";
				break;
			}
			case siena::string_id: {
				siena::string_t sval = iterator->string_value();
				cout << " - string(" << sval << ")";
				break;
			}
		}
		cout << " - done\n";
	} while (iterator->next());
*/
}
