// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Authors: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2002 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: sff.cc,v 1.2 2002/11/26 18:51:40 carzanig Exp $
//
#include <exception>
#include <iostream>
#include <cstring>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "yysff.h"

#include "callback.h"

using namespace std;

// --- extern global

extern FwdTable FT;

// ---

int print_usage(const char * progname) {
    cout << "Siena Fast Forwarding Mudule" << endl
	 << "(" << PACKAGE << " v." << VERSION << ")" << endl
	 << "copyright (C) 2001-2002 University of Colorado" << endl
	 << "usage: " << progname << "  -h | <filter file> [ <pcap file> ] " << endl;
}

int main(int argc, char *argv[]) 
{
    int res;
    const char* file = NULL;

    try {
	switch (argc) {
	case 3:
		file = argv[2];
	case 2:
	    if (argv[1][0] == '-' && argv[1][1] == 'h' && argv[1][2] == '\0') {
		print_usage(argv[0]);
		return 0;
	    }
	    if (sff_scanner_open(argv[1])) {
		cerr << "can't read " << argv[1] << endl;
		return 1;
	    }
	case 1: 
	    res = sff_parser_run();
		sffnmSetupCallback(&FT, file);
		sffnmPrintStats();
	    sff_scanner_close();
	    return res;

	default:
	    print_usage(argv[0]);
	    return 1;
	}
    } catch (exception & ex) {
	cout << ex.what() << endl;
    }
}
